%% Main Fuction %%
function skm_2dof_vrml

global Hv_vrml
 
%------- Figure settings 
figure('KeyPressFcn',@key_callback);

%------- Figure Settings
figure(1);

%-------- VRML Open 
Hv_vrml=vrworld('2-DOF.wrl');
open(Hv_vrml);
view(Hv_vrml);
 
%% Keyboard Interrrupt %%
function key_callback(obj,event)
global Hv_vrml
cla;                                % Erase the current figure %
persistent pos ang th
 
D2R=pi/180; R2D=180/pi;
dpos=5; dth=5*D2R;
a1=	100; a2=100;
param=[0,a1,0; 0,a2,0];
 
if isempty(pos), pos=[a1,a2]'; end
if isempty(ang), ang=90*D2R; end
if isempty(th), th=[0,90*D2R]'; end
 
key=event.Character;
switch key
    case '1', th(1,1)=th(1,1)+dth; index=1;        % +th1 Movement %
    case 'q', th(1,1)=th(1,1)-dth; index=1;        % -th1 Movement %
    case '2', th(2,1)=th(2,1)+dth; index=1;        % +th2 Movement %
    case 'w', th(2,1)=th(2,1)-dth; index=1;        % -th2 Movement %              
 
    case 'a', pos(1,1)=pos(1,1)+dpos; index=2;      % +X Movement %
    case 'z', pos(1,1)=pos(1,1)-dpos; index=2;      % -X Movement %
    case 's', pos(2,1)=pos(2,1)+dpos; index=2;      % +Y Movement %
    case 'x', pos(2,1)=pos(2,1)-dpos; index=2;      % -Y Movement %   
        
    case 'p', pos=[a1,a2]'; th=[0,90*D2R]'; index=1;   % Original Pos & Orien %        
    otherwise
        disp('Not Proper Key'); index=1;
end
 
switch index
    case 1, % Forward Kinematics %
        T02=forward_kinematics(th, param);
        pos=T02(1:2,4);
    case 2, % Inverse Kinematics %
        [th_c, w_index]=inverse_kinematics(pos, param);
        if w_index==1, th=th_c; end
end
        
% Draw the 2-DOF Planar Robot with VRML %
Hv_vrml.dad_Joint1.rotation = [0 1 0 th(1)];
Hv_vrml.dad_Joint2.rotation = [0 1 0 th(2)];

clc;
disp('px[mm], py[mm]');
disp(pos');
disp('th1[deg], th2[deg]');
disp(R2D*th');
disp('Transformation Matrix');
T02=forward_kinematics(th,param);
disp(T02);
 
%% Forward & Inverse Kinematics
%-------- Forward Kinematics
function T=forward_kinematics(th, param)
T=Transformation(th(1,1), param(1,:))*Transformation(th(2,1), param(2,:));
 
%-------- Inverse Kinematics
function [th, w_index]=inverse_kinematics(pos, param)
w_index=1;  % In workspace %
a1=param(1,2); a2=param(2,2);
 
% (1) Calculate th2 %
px=pos(1,1); py=pos(2,1);
kapha=(px^2+py^2-a1^2-a2^2)/(2*a1*a2);
if abs(kapha)>1
    w_index=-1;  % Out of Workspace %
    kapha=1;
end
th2=+acos(kapha); % Elbow Down %
%th2=-acos(kapha); % Elbow Up %
 
% (2) Calculate th1 %
delta=a1^2+a2^2+2*a1*a2*cos(th2);
cth1=(px*(a1+a2*cos(th2))+py*a2*sin(th2))/delta;
sth1=(-px*a2*sin(th2)+py*(a1+a2*cos(th2)))/delta;
th1=atan2(sth1, cth1); 
th=[th1, th2]';
 
%-------- D-H Transformation Matrix
function [T]=Transformation(th, param)
alpha=param(1); a=param(2); d=param(3);
T=Trans([0,0,d]')*Rotz(th)*Trans([a,0,0]')*Rotx(alpha);

%% Homogeneous Transformation Matrix %%
%------- Rotation matrix about the X-axis 
function [T]=Rotx(th)
cx=cos(th); sx=sin(th);
T=[1,0,0,0; 0,cx,-sx,0; 0,sx,cx,0; 0,0,0,1];
 
%------- Rotation matrix about the Y-axis 
function [T]=Roty(th)
cy=cos(th); sy=sin(th);
T=[cy,0,sy,0; 0,1,0,0; -sy,0,cy,0; 0,0,0,1];
 
%------- Rotation matrix about the Z-axis 
function [T]=Rotz(th)
cz=cos(th); sz=sin(th);
T=[cz,-sz,0,0; sz,cz,0,0; 0,0,1,0; 0,0,0,1];
 
%-------- Translation along the x, y, z axes 
function [T]=Trans(p)
T=[1,0,0,p(1,1); 0,1,0,p(2,1); 0,0,1,p(3,1); 0,0,0,1];
