%% Main Fuction %%
function skm_3dof
 
%------- Figure settings 
figure('KeyPressFcn',@key_callback);
 
%------- Figure Settings
%view(125,25); axis square
axis([-200, 200, -200, 200]);
xlabel('x-axis'); ylabel('y-axis');
grid on;
 
%% Keyboard Interrrupt %%
function key_callback(obj,event)
cla;                                % Erase the current figure %
persistent pos ang th
 
D2R=pi/180; R2D=180/pi;
dpos=5; dth=5*D2R;
a1=	100; a2=70; a3=50;
param=[0,a1,0; 0,a2,0;0,a3,0];
 
if isempty(pos), pos=[a1,a2+a3]'; end
if isempty(ang), ang=90*D2R; end
if isempty(th), th=[0,90*D2R,0]'; end
 
key=event.Character;
switch key
    case '1', th(1,1)=th(1,1)+dth; index=1;        % +th1 Movement %
    case 'q', th(1,1)=th(1,1)-dth; index=1;        % -th1 Movement %
    case '2', th(2,1)=th(2,1)+dth; index=1;        % +th2 Movement %
    case 'w', th(2,1)=th(2,1)-dth; index=1;        % -th2 Movement %
    case '3', th(3,1)=th(3,1)+dth; index=1;        % +th3 Movement %
    case 'e', th(3,1)=th(3,1)-dth; index=1;        % -th3 Movement %                
 
    case 'a', pos(1,1)=pos(1,1)+dpos; index=2;      % +X Movement %
    case 'z', pos(1,1)=pos(1,1)-dpos; index=2;      % -X Movement %
    case 's', pos(2,1)=pos(2,1)+dpos; index=2;      % +Y Movement %
    case 'x', pos(2,1)=pos(2,1)-dpos; index=2;      % -Y Movement %
    case 'd', ang=ang+dth; index=2;                 % +ANG Movement %
    case 'c', ang=ang-dth; index=2;                 % -ANG Movement %        
        
    case 'p', pos=[a1,a2+a3]'; ang=90*D2R; th=[0,90*D2R,0]'; index=1;   % Original Pos & Orien %        
    otherwise
        disp('Not Proper Key'); index=1;
end
 
% Draw the Fixed Frame %
sc_f=50; draw_frame(eye(4,4), sc_f);         % Fixed Frame %
 
switch index
    case 1, % Forward Kinematics %
        T03=forward_kinematics(th, param);
        pos=T03(1:2,4); ang=atan2(T03(2,1),T03(1,1));
    case 2, % Inverse Kinematics %
        [th_c, w_index]=inverse_kinematics(pos, ang,param);
        if w_index==1, th=th_c; end
end
        
% Draw a 3-DOF Planar Robot %
sc_r=100; draw_3dof_robot(th, param, sc_r);

clc;
disp('px[mm], py[mm], ang[deg]');
disp([pos(1,1), pos(2,1), ang*R2D]);
disp('th1[deg], th2[deg], th3[deg]');
disp([th(1,1), th(2,1), th(3,1)]*R2D);
disp('Transformation Matrix');
T03=forward_kinematics(th,param);
disp(T03);
 
%% Forward & Inverse Kinematics
%-------- Forward Kinematics
function T=forward_kinematics(th, param)
T=Transformation(th(1,1), param(1,:))*Transformation(th(2,1), param(2,:))*Transformation(th(3,1), param(3,:));
 
%-------- Inverse Kinematics
function [th, w_index]=inverse_kinematics(pos, ang, param)
w_index=1;  % In workspace %
a1=param(1,2); a2=param(2,2); a3=param(3,2); 
 
% (0) Calculate T03 %
T03=Trans([pos(1,1),pos(2,1),0]')*Rotz(ang);
 
% (1) Calculate th2 %
qx=T03(1,4); qy=T03(2,4);
cphi=T03(1,1); sphi=T03(2,1);
px=qx-a3*cphi; py=qy-a3*sphi;
kapha=(px^2+py^2-a1^2-a2^2)/(2*a1*a2);
if abs(kapha)>1
    w_index=-1;  % Out of Workspace %
    kapha=1;
end
th2=+acos(kapha); % Elbow Down %
%th2=-acos(kapha); % Elbow Up %
 
% (2) Calculate th1 %
delta=a1^2+a2^2+2*a1*a2*cos(th2);
cth1=(px*(a1+a2*cos(th2))+py*a2*sin(th2))/delta;
sth1=(-px*a2*sin(th2)+py*(a1+a2*cos(th2)))/delta;
th1=atan2(sth1, cth1);
 
% (3) Calcuate th3 %
phi=atan2(sphi, cphi);
th3=phi-(th1+th2);
 
th=[th1, th2, th3]';
 
%-------- D-H Transformation Matrix
function [T]=Transformation(th, param)
alpha=param(1); a=param(2); d=param(3);
T=Trans([0,0,d]')*Rotz(th)*Trans([a,0,0]')*Rotx(alpha);
 
%% Draw Primitives %%
%-------- Draw a 3-DOF Planar Robot
function draw_3dof_robot(th, param, sc)
% Draw the First Link %
T01=Transformation(th(1,1), param(1,:));
p1=[0,0,0]'; p2=T01(1:3,4); 
lw=10; clr=[0.5,0.1,0.1];
draw_line(p1,p2,lw,clr);
 
% Draw the Second Link %
T02=T01*Transformation(th(2,1), param(2,:));
p1=T01(1:3,4); p2=T02(1:3,4);
lw=10; clr=[0.1,0.5,0.1];
draw_line(p1,p2,lw,clr);
 
% Draw the Third Link %
T03=T02*Transformation(th(3,1), param(3,:));
p1=T02(1:3,4); p2=T03(1:3,4);
lw=10; clr=[0.1,0.1,0.5];
draw_line(p1,p2,lw,clr);
 
% Draw the Moving Frame & Box %
sc_f=50; draw_frame(T03, sc_f/5);  
sc_b=5; draw_box(T03, sc_b);

%-------- Draw a Frame
function draw_frame(T, sc)
p1=T*[0,0,0,1]'; p2=T*[sc,0,0,1]'; lw=2; clr=[1,0,0];
draw_line(p1,p2,lw,clr);
p1=T*[0,0,0,1]'; p2=T*[0,sc,0,1]'; lw=2; clr=[0,1,0];
draw_line(p1,p2,lw,clr);
p1=T*[0,0,0,1]'; p2=T*[0,0,sc,1]'; lw=2; clr=[0,0,1];
draw_line(p1,p2,lw,clr);
 
%------- Draw a Line
function draw_line(p1, p2, lw, clr)
line([p1(1,1),p2(1,1)],[p1(2,1),p2(2,1)],[p1(3,1),p2(3,1)],'LineWidth',lw,'Color',clr);
 
%------ Draw a Box %%% Multifaceted Patches
function draw_box(T, sc)
vt0=[0,0,0,1; sc,0,0,1; sc,sc,0,1; 0,sc,0,1; 0,0,sc,1; sc,0,sc,1; sc,sc,sc,1; 0,sc,sc,1]';
fm=[1,2,6,5; 2,3,7,6; 3,4,8,7; 4,1,5,8; 1,2,3,4; 5,6,7,8];
vt1=T*vt0; vm=vt1(1:3,:)';
patch('Vertices',vm,'Faces',fm,...
      'FaceVertexCData',gray(8),'FaceColor','flat');
 
%% Homogeneous Transformation Matrix %%
%------- Rotation matrix about the X-axis 
function [T]=Rotx(th)
cx=cos(th); sx=sin(th);
T=[1,0,0,0; 0,cx,-sx,0; 0,sx,cx,0; 0,0,0,1];
 
%------- Rotation matrix about the Y-axis 
function [T]=Roty(th)
cy=cos(th); sy=sin(th);
T=[cy,0,sy,0; 0,1,0,0; -sy,0,cy,0; 0,0,0,1];
 
%------- Rotation matrix about the Z-axis 
function [T]=Rotz(th)
cz=cos(th); sz=sin(th);
T=[cz,-sz,0,0; sz,cz,0,0; 0,0,1,0; 0,0,0,1];
 
%-------- Translation along the x, y, z axes 
function [T]=Trans(p)
T=[1,0,0,p(1,1); 0,1,0,p(2,1); 0,0,1,p(3,1); 0,0,0,1];
