% Main Fuction %
function fivebar_type1
 
%------- Figure settings 
figure('KeyPressFcn',@key_callback);
 
%------- Figure Settings
%view(125,25); 
axis square
axis([-200, 200, -50, 350]);
xlabel('x-axis'); ylabel('y-axis');
grid on;
 
% Keyboard Interrrupt %
function key_callback(obj,event)
cla;                                % Erase the current figure %
persistent pos th
 
D2R=pi/180; R2D=180/pi;
dpos=5; dth=5*D2R;
%----  L1, L2, L3, L4, L5, L6 ----%
param=[54,100,100,100,100,28];
 
if isempty(pos), pos=[48.5600, 168.2722]'; end
if isempty(th), th=D2R*[120,60]'; end
 
key=event.Character;
switch key
    case '1', th(1,1)=th(1,1)+dth; index=1;        % +th1 Movement %
    case 'q', th(1,1)=th(1,1)-dth; index=1;        % -th1 Movement %
    case '2', th(2,1)=th(2,1)+dth; index=1;        % +th2 Movement %
    case 'w', th(2,1)=th(2,1)-dth; index=1;        % -th2 Movement %
 
    case 'a', pos(1,1)=pos(1,1)+dpos; index=2;      % +X Movement %
    case 'z', pos(1,1)=pos(1,1)-dpos; index=2;      % -X Movement %
    case 's', pos(2,1)=pos(2,1)+dpos; index=2;      % +Y Movement %
    case 'x', pos(2,1)=pos(2,1)-dpos; index=2;      % -Y Movement %
        
    case 'p', pos=[48.5600, 168.2722]'; th=D2R*[120,60]'; index=1;   % Original Pos & Orien %        
    otherwise
        disp('Not Proper Key'); index=1;
end
 
% Draw the Fixed Frame %
%sc_f=50; draw_frame(eye(4,4), sc_f);         % Fixed Frame %

switch index
    case 1, % Forward Kinematics %
        pos=forward_kinematics(th, param);
    case 2, % Inverse Kinematics %
        [th_c, w_index]=inverse_kinematics(pos,param);
        if w_index==1, th=th_c; end
end
        
% Draw a 3-DOF Planar Robot %
draw_5bar(th, param);

clc; 
disp('px[mm], py[mm]');
disp([pos(1,1), pos(2,1)]);

disp('dx[mm], dy[mm]');
disp([pos(1,1)-48.5600, pos(2,1)-168.2722]);

disp('th1[deg], th2[deg]');
disp([th(1,1), th(2,1)]*R2D);

[JM]=Jacobian(pos, param);
disp('Jacobian Matrix');
disp(JM);
disp('Torque [Nmm]');
disp((inv(JM)*[10,0]')');

% Forward & Inverse Kinematics
%-------- Forward Kinematics
function [pos, w_index]=forward_kinematics(th, param)
persistent pos_p

if isempty(pos_p), pos_p=[48.5600, 168.2722]'; end

L1=param(1); L2=param(2); L3=param(3); L4=param(4); L5=param(5); L6=param(6); 
th2=th(1,1); th5=th(2,1);

% (1) 
pb=[L2*cos(th2); L2*sin(th2)];
pd=[L1+L5*cos(th5); L5*sin(th5)];
% (2)
[pc, w_index]=find_intersection(pb,L3,pd,L4, +1);
if w_index==1
    % (3)
    th3=atan2(pc(2,1)-pb(2,1), pc(1,1)-pb(1,1));
    % (4)
    pos=pc+L6*[cos(th3); sin(th3)];
else
    pos=pos_p;
end
 
% Backsubstitution %
pos_p=pos;

%-------- Inverse Kinematics
function [th, w_index]=inverse_kinematics(pos, param)
persistent th3_p th4_p
L1=param(1); L2=param(2); L3=param(3); L4=param(4); L5=param(5); L6=param(6); 
w_index=1;
if isempty(th3_p), th3_p=0; end
if isempty(th4_p), th4_p=0; end

% (1) Left Side 2-DOF Arm %
px=pos(1,1); py=pos(2,1);
k1=(px^2+py^2-L2^2-(L3+L6)^2)/(2*L2*(L3+L6));
if abs(k1)<=1
    th3=-acos(k1);
else
    w_index=-1;
    th3=th3_p;
end

DEL1=L2^2+(L3+L6)^2+2*L2*(L3+L6)*cos(th3);
cth2=(px*(L2+(L3+L6)*cos(th3))+py*(L3+L6)*sin(th3))/DEL1;
sth2=(-px*(L3+L6)*sin(th3)+py*(L2+(L3+L6)*cos(th3)))/DEL1;
th2=atan2(sth2,cth2);

% (2) Right Side 2-DOF Arm %
cx=px-L6*cos(th2+th3)-L1; cy=py-L6*sin(th2+th3);
k2=(cx^2+cy^2-L5^2-L4^2)/(2*L5*L4);
if abs(k2)<=1
    th4=acos(k2);
else
    w_index=-2;
    th4=th4_p;
end

DEL2=L5^2+L4^2+2*L5*L4*cos(th4);
cth5=(cx*(L5+L4*cos(th4))+cy*L4*sin(th4))/DEL2;
sth5=(-cx*L4*sin(th4)+cy*(L5+L4*cos(th4)))/DEL2;
th5=atan2(sth5,cth5);

th=[th2;th5];

th3_p=th3; th4_p=th4;

function [JM]=Jacobian(pos, param)

L1=param(1); L2=param(2); L3=param(3); L5=param(5);
%L4=param(4); L6=param(6); 

[th]=inverse_kinematics(pos, param);
th2=th(1,1); th5=th(2,1);

%pa=[0; 0];
pb=[L2*cos(th2); L2*sin(th2)];
pd=[L1+L5*cos(th5); L5*sin(th5)];
%pe=[L1;0];
th3=atan2(pos(2,1)-pb(2,1), pos(1,1)-pb(1,1));      % Absolute Angle %        
pc=pb+L3*[cos(th3); sin(th3)];
th4=atan2(pc(2,1)-pd(2,1), pc(1,1)-pd(1,1));        % Absolute Angle %

r1=L2*[cos(th2),sin(th2),0]';
r2=L5*[cos(th5),sin(th5),0]';
u1=[cos(th3),sin(th3),0]';
u2=[cos(th4),sin(th4),0]';
k=[0,0,1]';

d1=dot(cross(r1,u1),k);
d2=dot(cross(r2,u2),k);
JM=[u1(1:2,:),u2(1:2,:)]*inv(diag([d1,d2]));

%{
function [JM]=Jacobian(pos, param)
L1=param(1); L2=param(2); L3=param(3); L4=param(4); L5=param(5); L6=param(6); 

[th]=inverse_kinematics(pos, param);
th2=th(1,1); th5=th(2,1);

pa=[0; 0];
pb=[L2*cos(th2); L2*sin(th2)];
pd=[L1+L5*cos(th5); L5*sin(th5)];
pe=[L1;0];
th3=atan2(pos(2,1)-pb(2,1), pos(1,1)-pb(1,1));
pc=pb+L3*[cos(th3); sin(th3)];
th4=atan2(pc(2,1)-pd(2,1), pc(1,1)-pd(1,1));

sc=[cos(th2); sin(th2); (L3+L6)*sin(th3-th2)];
sa1=[cos(th3); sin(th3); 0];
sa2=[cos(th4); sin(th4); L6*sin(th3-th4)];
S2=[(pa(2,1)-pos(2,1)); -(pa(1,1)-pos(1,1)); 1];
S5=[(pe(2,1)-pos(2,1)); -(pe(1,1)-pos(1,1)); 1];

JM=[sa1/(sa1'*S2), sa2/(sa2'*S5), sc];
%}

% Find the intersection points of two circles %
function [pout, index]=find_intersection(c1,r1,c2,r2, selection)

th=atan2(c2(2,1)-c1(2,1),c2(1,1)-c1(1,1));
d=norm(c2-c1);
if d>abs(r1-r2) && d<abs(r1+r2)
    px1=(d^2+r1^2-r2^2)/(2*d);
    py1=selection*sqrt(r1^2-px1^2);
    index=1;
else
    px1=0;
    py1=0;
    index=-1;
end
pout=[cos(th)*px1-sin(th)*py1+c1(1,1);...
    sin(th)*px1+cos(th)*py1+c1(2,1)];
 
% Draw Primitives %
%-------- Draw a 5-bar Mechanism
function draw_5bar(th, param)

L1=param(1); L2=param(2); L3=param(3); L4=param(4); L5=param(5); L6=param(6); 
th2=th(1,1); th5=th(2,1);

% Forward Kinematics %
pa=[0;0];
pe=[L1;0];
pb=[L2*cos(th2); L2*sin(th2)];
pd=[L1+L5*cos(th5); L5*sin(th5)];
pc=find_intersection(pb,L3,pd,L4, +1);
th3=atan2(pc(2,1)-pb(2,1), pc(1,1)-pb(1,1));
pp=pc+L6*[cos(th3); sin(th3)];

lw=5; clr=[0.5,0.1,0.1];
% Draw the 1st Link %
draw_line(pa,pe,lw,clr);
% Draw the 2nd Link %
draw_line(pa,pb,lw,clr);
% Draw the 3rd Link %
draw_line(pb,pp,lw,clr);
% Draw the 4th Link %
draw_line(pc,pd,lw,clr);
% Draw the 5th Link %
draw_line(pd,pe,lw,clr);
 
% Draw the Moving Frame & Box %
draw_frame();  

%-------- Draw a Frame
function draw_frame()
p1=[0,0]'; p2=[50,0]'; lw=2; clr=[1,0,0];
draw_line(p1,p2,lw,clr);
p1=[0,0]'; p2=[0,50]'; lw=2; clr=[0,1,0];
draw_line(p1,p2,lw,clr);
 
%------- Draw a Line
function draw_line(p1, p2, lw, clr)
line([p1(1,1),p2(1,1)],[p1(2,1),p2(2,1)],'LineWidth',lw,'Color',clr);
 