% 2-DOF Serial Robot
% Trajectory Planning in the Cartesian Space

function []=dyn_2dof()
clear all;
global MAX_CH
MAX_CH=2;

a1=100; a2=100;
x0=[a1,a2]';
xf=[-a1,a2]';
tf=5; dt=0.001;

x=zeros(2,1); x_dot=zeros(2,1); x_ddot=zeros(2,1);
[c] = coeff_3rd(x0, zeros(MAX_CH,1), xf, zeros(MAX_CH,1), tf);

n=1;
for t=0:dt:tf   
    for ch=1:2
        x(ch,1)=c(ch,1)+c(ch,2)*t+c(ch,3)*t^2+c(ch,4)*t^3;
        x_dot(ch,1)=c(ch,2)+2*c(ch,3)*t+3*c(ch,4)*t^2;
        x_ddot(ch,1)=2*c(ch,3)+6*c(ch,4)*t;
    end       
    [th]=inverse_kinematics(x);
    [th_dot, th_ddot]=derivatives(th, dt);
    
    % Required Torques %
    [T]=dynamics(th_ddot, th_dot, th);
    x_data(n,1)=t;
    y_data1(n,:)=[x(1,1), x_dot(1,1), x_ddot(1,1)];
    y_data2(n,:)=T;
    n=n+1;
end
 
figure(1)
subplot(2,1,1), plot(x_data, y_data1);
xlabel('time [sec]'); ylabel('pos[mm], vel[mm/s], acc[mm/s^2]'); grid on;
subplot(2,1,2), plot(x_data, y_data2);
xlabel('time [sec]'); ylabel('Torque [Nmm]'); grid on;

%{
figure(2),
subplot(3,1,1), plot(x_data, -y_data1(:,1));
ylabel('Pos'); grid on;
subplot(3,1,2), plot(x_data, -y_data1(:,2));
ylabel('Vel'); grid on;
subplot(3,1,3), plot(x_data, -y_data1(:,3));
xlabel('time'); ylabel('Acc');
grid on;
%}

function [v, a]=derivatives(x, dt)
global MAX_CH
persistent xp vp

if isempty(xp), xp=x; end
if isempty(vp), vp=zeros(MAX_CH,1); end

v=(x-xp)/dt;     
a=(v-vp)/dt;   

% Back substitutions %
xp=x; vp=v;

function [th,w_index]=inverse_kinematics(xd)

w_index=1;  % In workspace %
a1=100; a2=100;

% (1) Calculate th2 %
px=xd(1,1); py=xd(2,1);
kapha=(px^2+py^2-a1^2-a2^2)/(2*a1*a2);
if abs(kapha)>1
    w_index=-1;  % Out of Workspace %
    kapha=1;
end
th2=+acos(kapha); % Elbow Down %
%th2=-acos(kapha); % Elbow Up %
 
% (2) Calculate th1 %
delta=a1^2+a2^2+2*a1*a2*cos(th2);
cth1=(px*(a1+a2*cos(th2))+py*a2*sin(th2))/delta;
sth1=(-px*a2*sin(th2)+py*(a1+a2*cos(th2)))/delta;
th1=atan2(sth1, cth1);
if th1<0, th1=th1+2*pi; end % 0<th<360 %

th=[th1, th2]';


function [T]=dynamics(th_ddot, th_dot, th)
% Robot Parameters %
m1=0.363; a1=100; ac1=87.74; Ic1=313.923;
m2=0.075; a2=100; ac2=41.68; Ic2=109.864;
g=9806.65;

% State Variables %
th1=th(1,1); th2=th(2,1);
th1_dot=th_dot(1,1); th2_dot=th_dot(2,1);
 
M(1,1)=m1*ac1^2+m2*(a1^2+ac2^2)+2*m2*a1*ac2*cos(th2)+Ic1+Ic2;
M(1,2)=m2*ac2^2+m2*a1*ac2*cos(th2)+Ic2;
M(2,1)=M(1,2);
M(2,2)=m2*ac2^2+Ic2;

C(1,1)=-m2*a1*ac2*sin(th2)*(2*th1_dot*th2_dot+th2_dot^2);
C(2,1)=m2*a1*ac2*sin(th2)*th1_dot^2;

G(1,1)=g*(m1*ac1*cos(th1)+m2*(a1*cos(th1)+ac2*cos(th1+th2)));
G(2,1)=m2*g*ac2*cos(th1+th2);
 
T=0.001*(M*th_ddot+C+G);

%---------------------- Trajectory Functions
function [c] = coeff_3rd(sp, sv, ep, ev, tsec)
MAX_CH=2;
c=zeros(MAX_CH,4);
for ch=1:MAX_CH
    c(ch,1)=sp(ch,1);
    c(ch,2)=sv(ch,1);
    c(ch,3)=(3.0*(ep(ch,1)-sp(ch,1))-(2.0*sv(ch,1)+ev(ch,1))*tsec)/(tsec*tsec);
    c(ch,4)=(-2.0*(ep(ch,1)-sp(ch,1))+(sv(ch,1)+ev(ch,1))*tsec)/(tsec*tsec*tsec);
end
