function [T, Y]=forward_2dof()
 
D2R=pi/180; R2D=180/pi;
 
tspan=[0,5];        % t0=0, tf=5 sec %
y0=D2R*[-30;0;0;0]; % th1_0=-30, th2_0=0, th1_dot_0=0, th2_dot_0=0 %
options = odeset('RelTol',1e-6,'AbsTol',[1e-6 1e-6 1e-6 1e-6]);
[T,Y] = ode45(@equation_of_motion,tspan,y0,options);
 
figure(1)
subplot(1,2,1), plot(T,Y(:,1)*R2D);
xlabel('time [sec]'); ylabel('th1 [deg]'); grid on;
subplot(1,2,2), plot(T,Y(:,2)*R2D);
xlabel('time [sec]'); ylabel('th2 [deg]'); grid on;
 
% simulation %
figure(2)
axis([-1.5, 1.5, -2, 1]); axis square
xlabel('x-axis'); ylabel('y-axis');
L1=1; L2=0.5; param=[0,L1,0; 0,L2,0];
size_Y=size(Y);
for n=1:size_Y(1)
    cla;    
    th(1,1)=Y(n,1); th(2,1)=Y(n,2);
    draw_2dof_robot(th, param);
    pause(0.02);
end
 
%% Dynamics
function dy=equation_of_motion(t, y)
 
% Robot Parameters %
m1=1; m2=0.5; L1=1; L2=0.5; g=9.81;
% State Variables %
th1=y(1,1); th2=y(2,1); th1_dot=y(3,1); th2_dot=y(4,1);
 
M(1,1)=((1/3)*m1+m2)*L1^2+m2*L1*L2*cos(th2)+(1/3)*m2*L2^2;
M(1,2)=(1/2)*m2*L1*L2*cos(th2)+(1/3)*m2*L2^2;
M(2,1)=M(1,2);
M(2,2)=(1/3)*m2*L2^2;
C(1,1)=-m2*L1*L2*sin(th2)*(th1_dot*th2_dot+(1/2)*th2_dot^2);
C(2,1)=(1/2)*m2*L1*L2*sin(th2)*th1_dot^2;
G(1,1)=g*(((1/2)*m1+m2)*L1*cos(th1)+(1/2)*m2*L2*cos(th1+th2));
G(2,1)=(1/2)*m2*g*L2*cos(th1+th2);
T(1,1)=0;
T(2,1)=0;
 
dy(1:2,1)=[th1_dot; th2_dot];
dy(3:4,1)=inv(M)*(-C-G+T);
 
%% Draw Functions
%-------- Draw a 3-DOF Planar Robot
function draw_2dof_robot(th, param)
% Draw the First Link %
T01=Transformation(th(1,1), param(1,:));
p1=[0,0,0]'; p2=T01(1:3,4); 
lw=4; clr=[0.5,0.1,0.1];
draw_line(p1,p2,lw,clr);
 
% Draw the Second Link %
T02=T01*Transformation(th(2,1), param(2,:));
p1=T01(1:3,4); p2=T02(1:3,4);
lw=4; clr=[0.1,0.5,0.1];
draw_line(p1,p2,lw,clr);

%-------- Draw a Frame
function draw_frame(T, sc)
p1=T*[0,0,0,1]'; p2=T*[sc,0,0,1]'; lw=2; clr=[1,0,0];
draw_line(p1,p2,lw,clr);
p1=T*[0,0,0,1]'; p2=T*[0,sc,0,1]'; lw=2; clr=[0,1,0];
draw_line(p1,p2,lw,clr);
p1=T*[0,0,0,1]'; p2=T*[0,0,sc,1]'; lw=2; clr=[0,0,1];
draw_line(p1,p2,lw,clr);
 
%------- Draw a Line
function draw_line(p1, p2, lw, clr)
line([p1(1,1),p2(1,1)],[p1(2,1),p2(2,1)],[p1(3,1),p2(3,1)],'LineWidth',lw,'Color',clr);
 
%------ Draw a Box %%% Multifaceted Patches
function draw_box(T, sc)
vt0=[0,0,0,1; sc,0,0,1; sc,sc,0,1; 0,sc,0,1; 0,0,sc,1; sc,0,sc,1; sc,sc,sc,1; 0,sc,sc,1]';
fm=[1,2,6,5; 2,3,7,6; 3,4,8,7; 4,1,5,8; 1,2,3,4; 5,6,7,8];
vt1=T*vt0; vm=vt1(1:3,:)';
patch('Vertices',vm,'Faces',fm,...
      'FaceVertexCData',gray(8),'FaceColor','flat');
 
%% Homogeneous Transformation Matrix %%
%------- Rotation matrix about the X-axis 
function [T]=Rotx(th)
cx=cos(th); sx=sin(th);
T=[1,0,0,0; 0,cx,-sx,0; 0,sx,cx,0; 0,0,0,1];
 
%------- Rotation matrix about the Y-axis 
function [T]=Roty(th)
cy=cos(th); sy=sin(th);
T=[cy,0,sy,0; 0,1,0,0; -sy,0,cy,0; 0,0,0,1];
 
%------- Rotation matrix about the Z-axis 
function [T]=Rotz(th)
cz=cos(th); sz=sin(th);
T=[cz,-sz,0,0; sz,cz,0,0; 0,0,1,0; 0,0,0,1];
 
%-------- Translation along the x, y, z axes 
function [T]=Trans(p)
T=[1,0,0,p(1,1); 0,1,0,p(2,1); 0,0,1,p(3,1); 0,0,0,1];

function [T]=Transformation(th, param)
alpha=param(1); a=param(2); d=param(3);
T=Trans([0,0,d]')*Rotz(th)*Trans([a,0,0]')*Rotx(alpha);

